//
//  RTC使用 ディジタルクロック (＋7segLED)
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//    RTC-8564、ASM1602(I2C)使用
//    RTCの1Hzクロックを外部割り込みで受けて1秒毎にLEDをON/OFFさせる
//    4桁7セグメントLED追加
//
//    12/05/14 T.Nakao
//

/*
  D2 RTC-CLOCK入力: 割込番号0

  スイッチ配置
           ●SW1(D3)
    ●SW4(D6)    ● SW2(D4)
           ●SW3(D5)

  SW1 UP
  SW3 DOWN
  SW2 SET
  SW4 SEL

*/

char *week[7] = {
  "SUN",
  "MON",
  "TUE",
  "WED",
  "THR",
  "FRY",
  "SAT"
};

#include <Wire.h>
#include <wI2cRtc8564.h>  // wI2cRtc8564
#include <wDisplay.h>     // wI2cLcd
#include <wSwitch.h>      // w4Switch

wI2cRtc8564 rtc;            // RTC-8564ドライバ
wI2cLcd lcd(LCD_8BITMODE);  // ACM1602
w4Switch sw;                // スイッチオブジェクトのインスタンス
wI2cD7S4Led d7sled;         // I2C化7セグメントLED インスタンス


byte Flag1Sec = false;
char StrBuf[17];

byte stHour, stMin, stSec;          // 設定用時刻バッファ
byte stYer, stMon, stDay, stWek;    // 設定用日付バッファ
byte Mode = 0;


//
// 初期化
//
void setup(void) {
  pinMode(13, OUTPUT);

  Wire.begin();           // I2Cマスタ初期化
  rtc.init();             // RTC初期化
  lcd.begin(16, 2);       // LCD初期化
  d7sled.begin(0x10);     // 7segLED

  sw.initSwitch(3, 4, 5, 6);   // SW用ポート定義
  sw.onKeyPress(KeyPress);     // スイッチ押下時のハンドラ登録
  sw.MaxCount = 100;

//  日時設定
//  rtc.setDate(0x12, 4, 0x13, 5);  // 2012/4/13 金
//  rtc.setTime(0x23, 0x59, 0x50);    //

  // 割込設定
  attachInterrupt(0, hSecInt, RISING);    // D2 立ち上がりエッジ
  rtc.setClkOut(1);       // RTC 1Hz Clock out enable

  d7sled.zeroPadding();
  d7sled.setDP(3);
}


//
// メインループ
//
void loop(void) {
  byte year, mon, day, wkday;
  byte hur, min, sec;

  sw.keyProc();                   // キーセンス処理

  // 1秒周期処理
  if(Flag1Sec) {
    Flag1Sec = false;

    if(Mode == 0) {
      rtc.readDate(&year, &mon, &day, &wkday);  // bcd
      rtc.readTime(&hur, &min, &sec);           // bcd

      lcd.setCursor(0, 0);
      sprintf(StrBuf, "20%02x/%02x/%02x %s", year, mon, day, week[wkday]);
      lcd.print(StrBuf);

      lcd.setCursor(0, 1);
      sprintf(StrBuf, "%02x:%02x:%02x", hur, min, sec);
      lcd.print(StrBuf);

      // 7セグメントLEDに時、分を表示
      // BCDコードを桁毎に分解して、桁毎に表示
      d7sled.putDigit( 3, (hur >> 4)   );   // 時(上位)
      d7sled.putDigit( 2, (hur & 0x0F) );   // 時(下位)
      d7sled.putDigit( 1, (min >> 4)   );   // 分(上位)
      d7sled.putDigit( 0, (min & 0xF)  );   // 分(下位)
    }
  }
}


//
// 割込ハンドラ(1Hz clock rising edge)
//
void hSecInt(void) {
  Flag1Sec = true;
}


//
// キー入力ハンドラ
//   スイッチが押されたときに呼び出される処理
//
void KeyPress(byte keyval) {
  switch(keyval) {
    case 1:     // UP
      KeyProcUp();
      break;
    case 2:     // SET
      KeyProcSet();
      break;
    case 3:     // DOWN
      KeyProcDown();
      break;
    case 4:     // SEL
      KeyProcSel();
      break;
  }
}


// 要BCD-BIN変換
void MenuDisp(void) {
  lcd.clear();

  switch(Mode) {
    case 0:     // 通常表示
//      lcd.noCursor();
      return;
    case 1:     // 設定モードの現在時刻設定[時]
      lcd.print("SET Hur:");
      sprintf(StrBuf, "%02d", stHour);
      lcd.print(StrBuf);
      break;
    case 2:     // 設定モードの現在時刻設定[分]
      lcd.print("SET Min:");
      sprintf(StrBuf, "%02d", stMin);
      lcd.print(StrBuf);
      break;
    case 3:     // 設定モードの現在時刻設定[秒]
      lcd.print("SET Sec:");
      sprintf(StrBuf, "%02d", stSec);
      lcd.print(StrBuf);
      break;
    case 4:     // 設定モードの現在日付設定[日]
      lcd.print("SET Day:");
      sprintf(StrBuf, "%02d", stDay + 1);
      lcd.print(StrBuf);
      break;
    case 5:     // 設定モードの現在日付設定[月]
      lcd.print("SET Mon:");
      sprintf(StrBuf, "%02d", stMon + 1);
      lcd.print(StrBuf);
      break;
    case 6:     // 設定モードの現在日付設定[年]
      lcd.print("SET Yer:");
      sprintf(StrBuf, "%02d", stYer);
      lcd.print(StrBuf);
      break;
    case 7:     // 設定モードの現在日付設定[曜日]
      lcd.print("SET WEK:");
      sprintf(StrBuf, "%s", week[stWek]);
      lcd.print(StrBuf);
      break;
  }
  lcd.setCursor(9, 0);
//  lcd.cursor();
}


// 項目選択
void KeyProcSel(void) {
  Mode = (Mode + 1) %  8;

  if(Mode == 1) {
    rtc.readDate(&stYer, &stMon, &stDay, &stWek, false);
    rtc.readTime(&stHour, &stMin, &stSec, false);

    // 0オリジンに修正
    stMon--;
    stDay--;
  }

  MenuDisp();
}


void KeyProcSet(void) {
  // 設定値確定
  rtc.setDate(stYer, stMon+1, stDay+1, stWek, false);
  rtc.setTime(stHour, stMin, stSec, false);

  Mode = 0;
  MenuDisp();
}


// UPキー
void KeyProcUp(void) {
  switch(Mode) {
    case 0:     // 通常表示
      return;
    case 1:     // 設定モードの現在時刻設定[時]
      stHour = (stHour + 1) % 24;
      break;
    case 2:     // 設定モードの現在時刻設定[分]
      stMin = (stMin + 1) % 60;
      break;
    case 3:     // 設定モードの現在時刻設定[秒]
      stSec = (stSec + 1) % 60;
      break;
    case 4:     // 設定モードの現在日付設定[日]
      stDay = (stDay + 1) % 31;
      break;
    case 5:     // 設定モードの現在日付設定[月]
      stMon = (stMon + 1) % 12;
      break;
    case 6:     // 設定モードの現在日付設定[年]
      stYer = (stYer + 1) % 100;
      break;
    case 7:     // 設定モードの現在日付設定[曜日]
      stWek = (stWek + 1) % 7;
      break;
  }
  MenuDisp();
}


// Downキー
// 要BCD-BIN変換
void KeyProcDown(void) {
  switch(Mode) {
    case 0:     // 通常表示
      return;
    case 1:     // 設定モードの現在時刻設定[時]
      stHour = (stHour + 23) % 24;
      break;
    case 2:     // 設定モードの現在時刻設定[分]
      stMin = (stMin + 59) % 60;
      break;
    case 3:     // 設定モードの現在時刻設定[秒]
      stSec = (stSec + 59) % 60;
      break;
    case 4:     // 設定モードの現在日付設定[日]
      stDay = (stDay + 30) % 31;
      break;
    case 5:     // 設定モードの現在日付設定[月]
      stMon = (stMon + 11) % 12;
      break;
    case 6:     // 設定モードの現在日付設定[年]
      stYer = (stYer + 99) % 100;
      break;
    case 7:     // 設定モードの現在日付設定[曜日]
      stWek = (stWek + 6) % 7;
      break;
  }
  MenuDisp();
}
